
Proot's backend takes care of creating cache files, cleaning them up, sending them to browsers and setting cache headers. This is done globally for all backend actions based on just a couple of settings.

Proot's caching system is very robust. Instead of scheduled jobs creating and destroying cache items in the background, generated responses from actions are saved as they are used, and also overwritten with newer versions automatically. If cache items are deleted for any reason, they are simply recreated while the action is used. If a cache item is unavailable or caching is broken for any reason, actions will still generate and output a response for the client.



## Cache files saved on the server (i.e. *backend cache*)

Proot can save the response generated by an action into a file. Next time a request comes in with the same parameters, this file will be shown instead of generating a new response, so long as the file isn't expired.

Caching behavior needs to be enabled **both** globally and by each action. The expiration time of cache files is set, in minutes, in `$settings['cache']` and `$action['cache']`. The shorter time of these two will be used.

Cached responses are saved in `cache/appname/actionparent/actionname/`. Cache directories are created automatically.

**Advanced usage:** the name of a cache file contains serialized input parameters in an encoded format. Use the function `cachename_decode()` to get these parameters in an array.



## Cache saved in user's browser (i.e. *browser cache*)

Proot will output valid cache headers with responses. These headers report an expiration time, after which the browser should make a new request to the server instead of remembering the previous one.

The expiration time is controlled both globally and by each action: Set a global maximum cache time, in minutes, in `$settings['browser cache']`; set your action's cache headers in `$action['browser cache']`.

Cache headers will be set to *no-store* in error responses.



## Disabling caching

Any time a cache time setting is set 0, that type of caching is disabled (either globally or for that action). The following is an example where caching is disabled in system settings when in debug mode.

#### `backend/hacks/after_settings.php`

	if ($settings['debug']) {

		// Make sure there's no caching
		$settings['cache'] = 0;
		$settings['browser cache'] = 0;

	} else {

		// Save cache files for a maximum of one week, send cache headers for 12 hours
		$settings['cache'] = 7*24*60;
		$settings['browser cache'] = 12;

	}
